// com.tts.initial.html5.textbutton
// HTML4 / HTML5: No differences
define( [
    "./accessibility.builder",
    "jquery"
    ], function (
        AccessibilityBuilder,
        jq
    ) {

    "use strict";

    var ButtonState = {
        Default: "btn_state_default",
        Hover: "btn_state_hover",
        Pressed: "btn_state_pressed"
    },

        ButtonBuilder = function (options) {

            var backgroundColors = {},
                textColors = {};

            backgroundColors = {};
            backgroundColors[ButtonState.Default] = options.color.background;
            backgroundColors[ButtonState.Hover] = options.color.backgroundHover;
            backgroundColors[ButtonState.Pressed] = options.color.backgroundPressed;

            textColors = {};
            textColors[ButtonState.Default] = options.color.text;
            textColors[ButtonState.Hover] = options.color.textHover;
            textColors[ButtonState.Pressed] = options.color.textPressed;

            function createTextContent() {

                if ((options.text.content|| "" ).trim() === "") {
                    return null;
                }

                var textContainer = jq(document.createElement("div"))
                    .css({
                        "width": options.width + "px",
                        "padding-right": options.textParams.paddingRight + "px",
                        "padding-left": options.textParams.paddingLeft + "px",
                        "text-align": options.textParams.orientation
                    });

                if (options.textParams.verticalAlignment === "middle") {
                    textContainer.css({
                        "vertical-align": "middle",
                        "display": "table-cell"
                    });
                }
                else {
                    textContainer.css({
                        "position": "absolute",
                        "padding-top": options.textParams.paddingTop + "px",
                        "padding-bottom": options.textParams.paddingBottom + "px"
                    });

                    textContainer.css(options.textParams.verticalAlignment, "0");

                    if (options.textParams.orientation === "right") {
                        textContainer.css(options.textParams.orientation, "0");
                    }
                }

                if (options.tooltip) {
                    textContainer
                        .attr({
                            "title": options.tooltip
                        })
                }

                textContainer.html(options.text.content);
                return textContainer;
            }

            function createButton() {

                var currentState = ButtonState.Default,
                    button = jq(document.createElement("button"))
                        .attr({
                            "class": currentState,
                            "role": "button"
                        })
                        .css({
                            "width": options.width + "px",
                            "height": options.height + "px",
                            "text-align": options.textParams.orientation,
                            "font-family": options.textParams.fontFamily,
                            "color": options.color.text,
                            "background-color": options.color.background,
                            "-webkit-tap-highlight-color": "rgba(0,0,0,0)",
                            "font-size": Math.min(Math.abs(options.textParams.fontSize), 100) + "px",
                            "cursor": "pointer",
                            "padding": "0",
                            "border": "none",
                            "background": "none"
                        })

                        .on("touchmove", function (e) {
                            e.preventDefault ? e.preventDefault() : true;
                        });

                if (options.textParams.verticalAlignment === "middle") {
                    button.css("display", "table");
                }

                if (options.ariaLabel || options.tooltip) {
                    button.attr({
                        "aria-label": options.ariaLabel ? options.ariaLabel : options.tooltip
                    })
                }

                if (options.tooltip) {
                    button.attr({
                        "title": options.tooltip
                    })
                }

                function changeState(buttonState) {
                    if (!buttonState) {
                        buttonState = currentState;
                    }

                    button.removeClass(currentState);
                    currentState = buttonState;
                    button.addClass(buttonState);

                    button.css({
                        "background-color": backgroundColors[buttonState],
                        "color": textColors[buttonState],
                        "cursor": "pointer"
                    });
                }

                function onBlur() {
                    var buttonState = ButtonState.Default;

                    changeState(buttonState);

                    button.off("focusout");
                    button.trigger("blur");
                }

                function onFocus() {
                    var buttonState = ButtonState.Hover;

                    changeState(buttonState);

                    button.trigger("focus");
                    button.on("focusout", onBlur);
                }

                button.append(createTextContent());

                return {
                    getRootElement: function () {
                        return button;
                    },
                    getHotspotElement: function () {
                        return button;
                    },
                    hasState: function (buttonState) {
                        return buttonState === currentState;
                    },
                    changeState: function (buttonState) {
                        changeState(buttonState);
                    },
                    focus: function () {
                        onFocus();
                    },
                    blur: function () {
                        onBlur();
                    }
                };
            }

            return {
                build: createButton
            };
        };

    /**
     * @constructor
     */
    return function (context, metadata) {
        var instance,
            shellElement,
            button,
            buttonInitialized = false,
            hasStageTriggerSupport = true,
            isInAuthoringMode = false,
            pointerUpEventName,
            pointerDownEventName,
            pointerMoveEventName,
            pointerLeaveEventName,
            isStandardPlayer = context.getPlayerName() === "standard",
            accessibilityTooltip,
            accessibilityTabOrder = metadata.parameters.asNumber("tab-order", -1),
            exludeFromTabControl = metadata.parameters.asBool("exclude-from-tab-control", false),
            accessibilityKey = metadata.parameters.get("accessibility-key", "KeyNone" ),
            accessibilityLocation = metadata.parameters.asNumber( "accessibility-location", 0 ),
            accessibilityPositionX = metadata.parameters.asNumber( "accessibility-position-x", -6 ),
            accessibilityPositionY = metadata.parameters.asNumber( "accessibility-position-y", -6 );

        function asRgbaValue(colorParameterName, defaultColor, transparencyParameterName) {

            var color = hexToRgb(metadata.parameters.get(colorParameterName, defaultColor)),
                alpha = asAlpha(metadata.parameters.asNumber(transparencyParameterName, 0));

            return "rgba(" + color.r + ", " + color.g + ", " + color.b + ", " + alpha + ")";
        }

        function hexToRgb(hex) {
            var parsingResult = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return parsingResult ? {
                r: parseInt(parsingResult[1], 16),
                g: parseInt(parsingResult[2], 16),
                b: parseInt(parsingResult[3], 16)
            } : {
                    r: 0,
                    g: 0,
                    b: 0
                };
        }

        function asAlpha(percentValue) {
            return (100 - percentValue) / 100;
        }

        // create an object shield so we are able to select and drag element
        function createAuthoringShield(shellElement) {
            shellElement.append(jq(document.createElement("div"))
                .css({
                    "position": "absolute",
                    "left": 0,
                    "top": 0,
                    "background-color": "#ffffff",
                    "opacity": "0.4",
                    "width": "100%",
                    "height": "100%",
                    "z-index": "9999"
                }));
        }

        function lockNavigation() {
			if (isInAuthoringMode || !hasStageTriggerSupport) {
                return;
            }
            context.stage.trigger.lock(metadata.getId());
        }

        function unlockNavigation() {
			if (isInAuthoringMode || !hasStageTriggerSupport) {
                return;
            }
            context.stage.trigger.unlock(metadata.getId());
        }

        function onMouseOver() {
            button.changeState(ButtonState.Hover);
            fireTriggerOnMouseOver();
        }

        function onMouseLeave() {
            button.changeState(ButtonState.Default);
            fireTriggerOnMouseLeave();
        }

        function onMouseDown(e) {
            cancelEvent(e);
            lockNavigation();
            button.changeState(ButtonState.Pressed);
        }

        function onMouseUp(e) {
            cancelEvent(e);
            unlockNavigation();

            button.changeState(ButtonState.Hover);

            if(!isOnTarget(e)) {
                return;
            }

            button.changeState(ButtonState.Default);
            fireTriggerOnUp();
        }

        function onAccessibilityKeyPressed() {
            button.changeState(ButtonState.Hover);
            button.changeState(ButtonState.Default);
            fireTriggerOnUp();
        }

        function cancelEvent(e){
            if ( isStandardPlayer ) {
                e.preventDefault();
                e.stopPropagation();
            }
        }

        function isOnTarget( e ) {
            var elem = e.target,
                origEvent = e.originalEvent,
                changedTouch;

            if ( !origEvent ) {
                return true;
            } else {
                changedTouch = origEvent.changedTouches && origEvent.changedTouches.length > 0 ? origEvent.changedTouches[0] : undefined;
            }

            if ( changedTouch ) {
                elem = document.elementFromPoint( changedTouch.clientX, changedTouch.clientY );
            }

            return e.target === elem;
        }

        function fireTriggerOnMouseOver(){
            setTimeout(function(){
                context.page.event.trigger(instance, "tg_mouseover");
            }, 10);
        }

        function fireTriggerOnMouseLeave(){
            setTimeout(function(){
                context.page.event.trigger(instance, "tg_mouseleave");
            }, 10);
        }

        function fireTriggerOnUp(){
            setTimeout(function(){
                context.page.event.trigger(instance, "tg_pressed");
            }, 10);
        }

        function initPointerEventNames() {
            if (!pointerUpEventName) {
                var nodeName = getHotspotElementNodeName();

                pointerUpEventName = context.device.getUpEvents(nodeName);
                pointerDownEventName = context.device.getDownEvents(nodeName);
                pointerMoveEventName = context.device.getMoveEvents(nodeName);
                pointerLeaveEventName = context.device.getReleaseEvents(nodeName);
            }
        }

        function getHotspotElementNodeName() {
            return "document";
        }

        function bindEvents() {

            if (context.stage.isInAuthoringMode()) {
                return;
            }

            if (!button || !buttonInitialized) {
                return;
            }

            context.page.interaction.provokesFeedback( button.getHotspotElement(), false );
            context.page.interaction.provokesAutosolve( button.getHotspotElement(), false );

            removeEvents();

            context.device.attachFirstSupportedEvent(button.getHotspotElement(), pointerDownEventName, onMouseDown);
            context.device.attachFirstSupportedEvent(button.getHotspotElement(), pointerMoveEventName, onMouseOver);
            context.device.attachFirstSupportedEvent(button.getHotspotElement(), pointerLeaveEventName, onMouseLeave);
            context.device.attachFirstSupportedEvent(button.getHotspotElement(), pointerUpEventName, onMouseUp);
        }

        function removeEvents() {
            unlockNavigation();
            if (!button || !buttonInitialized) {
                return;
            }

            initPointerEventNames();
            context.device.removeFirstSupportedEvent(button.getHotspotElement(), pointerDownEventName, onMouseDown);
            context.device.removeFirstSupportedEvent(button.getHotspotElement(), pointerMoveEventName, onMouseOver);
            context.device.removeFirstSupportedEvent(button.getHotspotElement(), pointerLeaveEventName, onMouseLeave);
            context.device.removeFirstSupportedEvent(button.getHotspotElement(), pointerUpEventName, onMouseUp);
        }

        function getText( parameterName ) {
            var paramValue = ( metadata.parameters.get(parameterName,"") || "" ).trim();
			return ( paramValue.length > 0 ) ? paramValue : null;
        }

        function initializeButton(){
            if( buttonInitialized ){
                return;
            }
            buttonInitialized = true;

            shellElement.css({
                "width": metadata.getWidth() + "px",
                "height": metadata.getHeight() + "px"
                });

            button = createButton();

            shellElement.append( button.getRootElement() );

            accessibilityTooltip = new AccessibilityBuilder(accessibilityKey, accessibilityLocation, accessibilityPositionX, accessibilityPositionY, context).build();
            if (accessibilityTooltip.getElement()) {
                shellElement.append(accessibilityTooltip.getElement());
            }

            context.page.interaction.provokesFeedback( button.getHotspotElement(), false );
            context.page.interaction.provokesAutosolve( button.getHotspotElement(), false );

            if (isInAuthoringMode) {
                createAuthoringShield( shellElement );
            }

            button.changeState( ButtonState.Default);
        }

        function createButton(){
            var scParams = metadata.parameters,
                button = new ButtonBuilder({
                    width: metadata.getWidth(),
                    height: metadata.getHeight(),
                    tooltip: getText("tooltip"),
                    instance: instance,
                    ariaLabel: ((metadata.parameters.get("aria-label", false)|| "" ).trim() === "") ? false : metadata.parameters.get("aria-label", false),
                    color : {
                        background: asRgbaValue("background-color", "#999999", "background-color-transparency"),
                        backgroundHover: asRgbaValue("background-color-hover", "#adadad", "background-color-hover-transparency"),
                        backgroundPressed: asRgbaValue("background-color-pressed", "#fd7326", "background-color-pressed-transparency"),
                        text : scParams.get("text-color", "#000000"),
                        textHover : scParams.get("text-color-hover", "#000000"),
                        textPressed : scParams.get("text-color-pressed", "#000000")
                    },
                    text : {
                        content : scParams.get("text", "Text")
                    },
                    textParams : {
                        fontFamily : getFontStyle( scParams.asNumber("font-family", "1") ),
                        fontSize : scParams.asNumber("font-size", "14"),
                        orientation : scParams.get("text_orientation", "center"),
                        verticalAlignment: scParams.get("text_vertical_align", "middle"),
                        paddingLeft : Math.abs(scParams.asNumber("padding-left", "10")),
                        paddingRight : Math.abs(scParams.asNumber("padding-right", "10")),
                        paddingTop : Math.abs(scParams.asNumber("padding-top", "10")),
                        paddingBottom : Math.abs(scParams.asNumber("padding-bottom", "10"))
                    }
                }).build();

            return button;
        }

        function getFontStyle( paramValue ) {
            switch( paramValue ) {
                case 2:
                    return "\"Times New Roman\", Times, serif";
                case 3:
                    return "Verdana, Geneva, sans-serif";
                case 4:
                    return "\"Courier New\", Courier, monospace";
                default:
                    return "Arial, Helvetica, sans-serif";
            }
        }

        // revealing public API
        return {
            onLoading: function () {
                instance = this;

                context.page.event.bindOnEnter(instance);
                context.page.event.bindOnExit(instance);
                context.stage.accessibility.bindOnAccessibilityModeChanged(instance);
                if (!isStandardPlayer) {
                    context.page.event.bind(instance, "onTabNavigationOff");
                    context.page.event.bind(instance, "onTabNavigationSelection");
                    context.page.event.bind(instance, "onTabNavigationFire");
                }

                hasStageTriggerSupport = !!context.stage.trigger;
                isInAuthoringMode = !!context.stage.isInAuthoringMode();

                shellElement = jq(context.getElement());

                initializeButton();
            },

            onEnterPage: function () {
                bindEvents();

                if (!isStandardPlayer && !exludeFromTabControl) {
                    accessibilityTooltip.registerTabNavigation(metadata.getId(), accessibilityTabOrder);
                }
            },

            onExitPage: function() {
                removeEvents();
            },

            onAccessibilityModeChanged: function ( dataHTML5, dataHTML4 ) {
                var params = dataHTML4 || dataHTML5;

                if ( params.isActive ) {
                    accessibilityTooltip.show();
                } else {
                    accessibilityTooltip.hide();
                }
            },

            onAccessibilityKeyPressed: function ( dataHTML5, dataHTML4 ) {
                var params = dataHTML4 || dataHTML5;

                if ( params.keyCode == accessibilityTooltip.getKeyCode() ) {
                    onAccessibilityKeyPressed();
                    context.stage.accessibility.turnAccessibilityModeOff();
                }
            },

            onTabNavigationSelection: function (dataHTML5, dataHTML4, data) {
                if (data && data.id && data.id == metadata.getId()) {
                    button.trigger("focus");
                } else {
                    button.trigger("blur");
                }
            },

            onTabNavigationOff: function (dataHTML5, dataHTML4) {
                button.trigger("blur");
            },

            onTabNavigationFire: function (dataHTML5, dataHTML4, data) {
                if (data && data.id && data.id == metadata.getId()) {
                    onAccessibilityKeyPressed();
                }
            }
        };
    };
});
